import {Component, OnDestroy, OnInit} from '@angular/core';
import {Chart} from 'angular-highcharts';
import {LecturerInfoService} from 'src/app/services/lecturer-info.service';
import {ResponseGraphDetailsDao} from 'src/app/dao/ResponseGraphDetails.dao';
import {LecturerResponseGraphService} from 'src/app/services/lecturer-response-graph.service';

@Component({
  selector: 'app-response-chart',
  templateUrl: './response-chart.component.html',
})
export class ResponseChartComponent implements OnInit, OnDestroy {

  public responseChart: any;
  public classesByYear: ResponseGraphDetailsDao[][];
  public isLoading: boolean;
  public years;
  public currentYear = 0;
  private lecturerId: string;

  constructor(private lecturerInfoService: LecturerInfoService,
              private lecturerGraphService: LecturerResponseGraphService) {
  }

  ngOnInit(): void {
    this.lecturerId = sessionStorage.getItem('username');
    this.isLoading = true;
    this.years = new Set();
    this.classesByYear = [];
    this.lecturerInfoService.getClassResponseData(this.lecturerId).subscribe((data: ResponseGraphDetailsDao[]) => {
      this.sortData(data);
      this.lecturerGraphService.changeClasses(this.classesByYear[0]);
      this.buildChart(0);
      this.isLoading = false;
    });
  }

  ngOnDestroy(): void {
    this.years.clear();
    this.classesByYear = [];
  }

  public switchYears() {
    let i = 0;
    this.classesByYear.forEach(elem => {
      if (elem[0].year === this.currentYear) {
        this.lecturerGraphService.changeYear(this.currentYear);
        this.lecturerGraphService.changeClasses(this.classesByYear[i]);
        this.buildChart(i);
        return;
      }
      i++;
    });
  }

  private sortData(data: ResponseGraphDetailsDao[]) {
    data.forEach(e => {
      this.years.add(e.year);
    });
    this.currentYear = this.years[0];

    this.years.forEach(year => {
      this.classesByYear.push(data.filter((elem: ResponseGraphDetailsDao) => elem.year === year));
    });
  }

  private getSubmittedSeries(data: ResponseGraphDetailsDao[]): any {
    const submitted = [];
    data.forEach(elem => {
      submitted.push(elem.numberSubmitted);
    });
    return {name: 'Submitted', data: submitted};
  }

  private getPendingSeries(data: ResponseGraphDetailsDao[]): any {
    const pending = [];
    data.forEach(elem => {
      pending.push(elem.numberPending);
    });
    return {name: 'Pending', data: pending};
  }

  private createXAxisHeaders(data: ResponseGraphDetailsDao[]): string[] {
    const xAxisHeaders = [];
    data.forEach(elem => {
      xAxisHeaders.push(elem.classCode.toString());
    });
    return xAxisHeaders;
  }

  private buildChart(index: number): void {
    this.responseChart = new Chart({
      chart: {
        type: 'column'
      },
      title: {
        text: 'Total Student Responses For Each Class'
      },
      xAxis: {
        categories: this.createXAxisHeaders(this.classesByYear[index]),
        title: {
          text: 'Classes In The Year'
        }
      },
      yAxis: {
        min: 0,
        title: {
          text: 'Number of Responses'
        }
      },
      credits: {
        enabled: false
      },
      series: [this.getSubmittedSeries(this.classesByYear[index]), this.getPendingSeries(this.classesByYear[index])]
    });
  }
}
